import java.awt.Color;
import java.util.HashSet;
import java.util.Set;
import java.util.Stack;

import com.collegeboard.gridworld.actor.Actor;
import com.collegeboard.gridworld.grid.Location;

public class Searcher extends Actor
{
   private Stack<Location> searchStack;
   private Set<Location> beenThere;
   private boolean firstStep;
   private boolean done;

   public Searcher()
   {
      this(Color.RED);
   }

   public Searcher(Color color)
   {
      setColor(color);

      searchStack = new Stack<Location>();
      beenThere = new HashSet<Location>();
      firstStep = true;
      done = false;
      turn(270);
   }

   public void act()
   {
      if(!done)
      {
         Food lunch = findFood();
         if(lunch != null)
         {
            eat(lunch);
            done = true;
         }
         else
            searchStep();
      }
   }

   public Food findFood()
   {
      for(int angle = 0; angle < 360; angle += 45)
      {
         Location loc = getLocation().getNeighborLocation(angle);
         if(getGrid().isValid(loc) &&
                      getGrid().get(loc) instanceof Food)
            return (Food)getGrid().get(loc);
      }
      return null;
   }

   public void eat(Food lunch)
   {
      Location loc = lunch.getLocation();
      setColor(lunch.getColor());
      moveTo(loc);
   }

   public void searchStep()
   {
      if(firstStep)
      {
         searchStack.push(getLocation());
         beenThere.add(getLocation());
         firstStep = false;
      }

      if(!searchStack.isEmpty())
      {
         moveTo((Location)searchStack.pop());

         for(int angle = 0; angle < 360; angle += 45)
         {
            Location loc = getLocation().getNeighborLocation(angle);
            if(getGrid().isValid(loc) &&
                         getGrid().get(loc) == null &&
                                 !beenThere.contains(loc))
            {
               beenThere.add(loc);
               searchStack.push(loc);
            }
         }
      }
   }

}


