import java.awt.Color;
import java.util.LinkedList;
import java.util.Set;
import java.util.HashSet;
import java.util.Queue;

import com.collegeboard.gridworld.grid.Location;
import com.collegeboard.gridworld.actor.Actor;

public class SearcherQueue extends Actor
{
   private Queue<Location> searchQ;
   private Set<Location> beenThere;
   private boolean firstStep;
   private boolean done;

   public SearcherQueue()
   {
      this(Color.RED);
   }

   public SearcherQueue(Color color)
   {
      setColor(color);

      searchQ = new LinkedList<Location>();
      beenThere = new HashSet<Location>();
      firstStep = true;
      done = false;
   }

   public void act()
   {
      if(!done)
      {
         Food lunch = findFood();
         if(lunch != null)
         {
            eat(lunch);
            done = true;
         }
         else
            searchStep();
      }
   }

   public Food findFood()
   {
      for(int angle = 0; angle < 360; angle += 45)
      {
         Location loc = getLocation().getNeighborLocation(angle);
         if(getGrid().isValid(loc) &&
                      getGrid().get(loc) instanceof Food)
            return (Food)getGrid().get(loc);
      }
      return null;
   }

   public void eat(Food lunch)
   {
      Location loc = lunch.getLocation();
      setColor(lunch.getColor());
      moveTo(loc);
   }

   public void searchStep()
   {
      if(firstStep)
      {
         searchQ.add(getLocation());
         beenThere.add(getLocation());
         firstStep = false;
      }

      if(!searchQ.isEmpty())
      {
         moveTo((Location)searchQ.remove());

         for(int angle = 0; angle < 360; angle += 45)
         {
            Location loc = getLocation().getNeighborLocation(angle);
            if(getGrid().isValid(loc) &&
                         getGrid().get(loc) == null &&
                                 !beenThere.contains(loc))
            {
               beenThere.add(loc);
               searchQ.add(loc);
            }
         }
      }
   }



}


