package com.collegeboard.gridworld.actor;

import java.util.ArrayList;

import com.collegeboard.gridworld.grid.Grid;
import com.collegeboard.gridworld.grid.Location;
import com.collegeboard.gridworld.world.World;

/**
 * A world that is occupied by actors.
 *
 * <p>
 * copyright&copy; 2005 Cay S. Horstmann (http://horstmann.com)
 *
 * @author Cay Horstmann
 */

public class ActorWorld extends World<Actor>
{
   public ActorWorld()
   {
      init();
   }
   
   public ActorWorld(Grid<Actor> grid)
   {
      super(grid);
      init();
   }
     
   private void init()
   {
      addOccupantClass("com.collegeboard.gridworld.actor.Rock");
   }
   
   public void step()
   {
      Grid<Actor> gr = getGrid();
      ArrayList<Actor> actors = new ArrayList<Actor>();
      for (Location loc : gr.getOccupiedLocations())
         actors.add(gr.get(loc));
      
      for (Actor a : actors)
         a.act();
   }

   public void add(Location loc, Actor occupant)
   {
      occupant.putSelfInGrid(getGrid(), loc);
   };
   
   public Actor remove(Location loc)
   {
      Actor occupant = getGrid().get(loc);
      if (occupant == null) return null;
      occupant.removeSelfFromGrid();
      return occupant;
   }
}