package com.collegeboard.gridworld.actor;


/**
 * A critter is an actor that makes simple moves and turns.
 * 
 *  <p>
 *  copyright&copy; 2005 Cay S. Horstmann (http://horstmann.com)
 *
 * @author Cay Horstmann
 */


import java.awt.Color;
import com.collegeboard.gridworld.grid.Location;

public class Critter extends Actor
{
   private boolean tracing;

   /**
    * Constructs a green critter that drops flowers as it moves.
    */
   public Critter()
   {
      setColor(Color.GREEN);
      tracing = true;
   }

   /**
    * Constructs a critter of a given color that drops flowers as it moves.
    * 
    * @param color the color for this critter
    */
   public Critter(Color color)
   {
      setColor(color);
      tracing = true;
   }

   public void act()
   {
      if (canMove())
         move();
      else
         turn();
   }

   /**
    * Turns 45 degrees to the right.
    */
   public void turn()
   {
      turn(Location.HALF_RIGHT);
   }

   /**
    * Moves in the current direction. If tracing is turned on, drops a flower in
    * the old location.
    */
   public void move()
   {
      Location forward = getLocation().getNeighborLocation(getDirection());

      Location old = getLocation();
      moveTo(forward);
      if (tracing)
      {
         Flower flower = new Flower(getColor());
         flower.putSelfInGrid(getGrid(), old);
      }
   }

   /**
    * Checks whether this critter can move in the current direction.   
    * @return true if the critter can move, false if the next location is
    *         invalid or occupied by an actor other than a flower
    */
   public boolean canMove()
   {
      Location forward = getLocation().getNeighborLocation(getDirection());
      if (!getGrid().isValid(forward))
         return false;
      Actor neighbor = getGrid().get(forward);
      return neighbor == null || neighbor instanceof Flower;
   }

   public void setTracing(boolean b)
   {
      tracing = b;
   }
}
