package com.collegeboard.gridworld.actor;

import java.awt.Color;


/**
 * Some actors drop flowers as they move. Flowers wilt by turning
 * black over time.
 *
 *  <p>
 *  copyright&copy; 2005 Cay S. Horstmann (http://horstmann.com)
 *
 * @author Cay Horstmann
 */

public class Flower extends Actor
{
   private static final Color DEFAULT_COLOR = Color.PINK;
   private static final double WILT_FACTOR = 0.05; 
      // lose 5% of color value in each step  
   
   /**
    * Constructs a pink flower.
    */
   public Flower()
   {
      setColor(DEFAULT_COLOR);
   }

   /**
    * Constructs a flower of a given color.
    * @param color the initial color of this flower
    */
   public Flower(Color color)
   {
      setColor(color);
   }

   /**
    * When a flower acts, it "wilts", gradually losing its color.
    */
   public void act()
   {
      Color c = getColor();
      int red = (int) (c.getRed() * (1 - WILT_FACTOR));
      int green = (int) (c.getGreen() * (1 - WILT_FACTOR));
      int blue = (int) (c.getBlue() * (1 - WILT_FACTOR));
      
      setColor(new Color(red, green, blue));
   }
}
