package com.collegeboard.gridworld.grid;

/**
 *  <code>Grid</code> provides an interface for a two-dimensional,
 *  grid-like environment containing arbitrary objects.  For example,
 *  it could be a grid of actors for a simulation.
 *
 *  copyright&copy; 2002 College Entrance Examination Board
 *  (www.collegeboard.com).
 *  copyright&copy; 2005 Cay S. Horstmann (www.horstmann.com)
 *  @author Alyce Brady
 *  @author APCS Development Committee
 *  @author Cay S. Horstmann
 **/

import java.util.ArrayList;

public interface Grid<E>
{
   /** Returns number of rows in this grid.
    *  @return the number of rows, or -1 if the grid is unbounded
    **/
   int numRows();
   
   /** Returns number of columns in this grid.
    *  @return the number of columns, or -1 if the grid is unbounded
    **/
   int numCols();
   
   /** Adds a new object to this grid.
    *  @param loc the location at which to put the object (must be valid)
    *  @param obj the new object to be added (must not be null)
    *  @return the object that was removed, or null if the location was previously
    *  unoccupied.
    **/
   E put(Location loc,  E obj);
   
   /** Removes an object from this grid.
    *  @param loc the location of the object that is to be removed (must be valid)
    *  @return the object that was removed, or null if no object was in
    *  the given location
    **/
   E remove(Location loc);
   
   /** Returns the object at a specific location in this grid 
    *  @param loc    the location in which to look (must be valid)
    *  @return       the object at location <code>loc</code>;
    *                <code>null</code> if <code>loc</code> is empty
    **/
   E get(Location loc);
   
   /** Gets the locations in this grid that contain objects.
    *  @return an array of all occupied locations in this grid
    **/
   ArrayList<Location> getOccupiedLocations();
   
   /** Checks whether a location is valid in this grid.
    *  @param  loc    location to check (must not be null)
    *  @return <code>true</code> if <code>loc</code> is valid in this grid;
    *          <code>false</code> otherwise
    **/
   boolean isValid(Location loc);
 }
