package com.collegeboard.gridworld.gui;



import java.awt.Color;
import java.awt.Font;
import java.awt.Component;
import java.awt.font.FontRenderContext;
import java.awt.font.LineMetrics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.geom.Rectangle2D;
import java.awt.Rectangle;

/**
 *  AP&reg; Computer Science Marine Biology Simulation:<br>
 *  A <code>DefaultDisplay</code> draws a centered question-mark.
 *  A <code>DefaultDisplay</code> object is used to display any
 *  locatable object that doesn't have a specialized display object.
 *  The association between a particular <code>Locatable</code> 
 *  subclass and its display object is handled in the
 *  <code>DisplayMap</code> class.
 *
 *  <p>
 *  The <code>DefaultDisplay</code> class is
 *  copyright&copy; 2002 College Entrance Examination Board
 *  (www.collegeboard.com).
 *
 *  @author Alyce Brady
 *  @author Jeff Raab, Northeastern University
 *  @version 1 August 2002
 *  @see Grid
 **/
public class DefaultDisplay implements Display
{
    /** Draw the given object.
     *  This implementation draws a question mark using the Java
     *  2D Graphics API.
     *  @param   obj        object we want to draw
     *  @param   comp       component on which to draw
     *  @param   g2         drawing surface
     *  @param   rect       rectangle in which to draw 
     **/
    public void draw(Object obj, Component comp, Graphics2D g2, Rectangle rect)
    {
        // Set rendering hints.
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                            RenderingHints.VALUE_ANTIALIAS_ON);

        
        Color color = (Color) AbstractDisplay.getProperty(obj, "color");
        if (color != null)
        {
           g2.setPaint(color);
           g2.fill(rect);
        }
        String text = (String) AbstractDisplay.getProperty(obj, "text");
        if (text == null) text = "?";
        g2.setPaint(Color.BLACK);
        g2.setFont(new Font("SansSerif", Font.BOLD, (int) (0.8 * rect.height)));
        // Paint it centered in the rectangle. 
        paintCenteredText(g2, text, rect.x + rect.width/2, rect.y + rect.height/2);
    }

    /** Paint a horizontally and vertically-centered text string.
     *  This method is adapted from p. 134 of J. Knudsen's book, Java 2D Graphics,
     *  published by O'Reilly & Associates (1999).
     *  @param g2           drawing surface
     *  @param s            string to draw (centered)
     *  @param centerX      x-coordinate of center point
     *  @param centerY      y-coordinate of center point 
     **/
    protected void paintCenteredText(Graphics2D g2,
                            String s, float centerX, float centerY)
    {
        FontRenderContext frc = g2.getFontRenderContext();
        Rectangle2D bounds = g2.getFont().getStringBounds(s, frc);
        float leftX = centerX -  (float)bounds.getWidth()/2;
        LineMetrics lm = g2.getFont().getLineMetrics(s, frc);
        float baselineY = centerY - lm.getHeight()/2 + lm.getAscent();
        g2.drawString(s, leftX, baselineY);
    }

}
