package com.collegeboard.gridworld.gui;



import java.awt.Graphics2D;
import java.awt.Component;
import java.awt.Rectangle;

/**
 *  AP&reg; Computer Science Marine Biology Simulation:<br>
 *  The <code>LocatableDisplay</code> interface contains the
 *  method needed to display an environment object.
 *  Objects that implement the <code>LocatableDisplay</code> interface are called
 *  on by the <code>EnvDisplay</code> to draw environment objects. The association between a 
 *  particular <code>Locatable</code> subclass and its display is handled
 *  in the <code>DisplayMap</code> class.
 *
 *  <p>
 *  The <code>LocatableDisplay</code> class is
 *  copyright&copy; 2002 College Entrance Examination Board
 *  (www.collegeboard.com).
 *
 *  @author Alyce Brady
 *  @version 1 August 2002
 *  @see Grid
 **/
public interface Display
{
    /** Method invoked to draw a Locatable. The first argument is the
     *  locatable object to draw, the second the component, the third the
     *  drawing surface, the last is the rectangle in which to draw. A
     *  class that implements this interface should draw a representation of
     *  the given Locatable object on the drawing surface in the given rectangle.
     *
     *  @param   obj        object we want to draw
     *  @param   comp       component on which to draw
     *  @param   g2         drawing surface
     *  @param   rect       rectangle in which to draw 
     **/
    void draw(Object obj, Component c, Graphics2D g2, Rectangle rect);

}
