package com.collegeboard.gridworld.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.util.Set;
import java.util.TreeSet;

import javax.swing.AbstractButton;
import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.filechooser.FileView;

import com.collegeboard.gridworld.actor.Actor;
import com.collegeboard.gridworld.actor.ActorWorld;
import com.collegeboard.gridworld.grid.Grid;
import com.collegeboard.gridworld.grid.Location;
import com.collegeboard.gridworld.world.World;

import java.io.File;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.net.URLClassLoader;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import javax.swing.JFileChooser;

/**
 * Copyright&copy; 2002 College Entrance Examination Board (www.collegeboard.com).
 * Copyright&copy; 2005 Cay S. Horstmann (http://horstmann.com)
 * 
 * @author Julie Zelenski
 * @author Chris Nevison
 * @author Cay Horstmann
 */
public class WorldFrame extends JFrame
{
   private SimController simControl;
   private GridPanel display;
   private JTextArea messageArea;
   private ArrayList<JMenuItem> menuItemsDisabledDuringRun;
   private World world;
   private ResourceBundle resources;
   private DisplayMap displayMap;

   private JFileChooser fileChooser;
   private Set<Class> gridClasses;
   private JMenu newGridMenu;
   
   private static int count = 0;
   
   public WorldFrame(World world)
   {
      this(world, world.getClass().getClassLoader());
   }
   
   public WorldFrame(World world,  ClassLoader loader)   
   {
      count++;
      resources = ResourceBundle.getBundle(getClass().getName() + "Resources");

      try
      {
         System.setProperty("sun.awt.exception.handler", GUIExceptionHandler.class.getName());
         configureFileChooser();
      }
      catch (SecurityException ex)
      {
         // these will fail in an applet
      }
      
      addWindowListener(new 
            WindowAdapter()
            {
               public void windowClosing(WindowEvent event)
               {
                  count--;
                  if (count == 0)
                     System.exit(0);
               }
            });

      displayMap = new DisplayMap();

      setTitle(resources.getString("frame.title"));
      setLocation(25, 15);

      makeMenus();
      
      JPanel content = new JPanel();
      content.setBorder(BorderFactory.createEmptyBorder(15, 15, 15, 15));
      content.setLayout(new BorderLayout());
      setContentPane(content);

      display = new GridPanel(displayMap, resources);
      JScrollPane scrollPane = new JScrollPane();
      scrollPane.setViewport(new PseudoInfiniteViewport(scrollPane));
      scrollPane.setViewportView(display);
      content.add(scrollPane, BorderLayout.CENTER);

      this.world = world;
      gridClasses = new TreeSet<Class>(new 
            Comparator<Class>()
            {
               public int compare(Class a, Class b)
               {
                  return a.getName().compareTo(b.getName());
               }
            });
      World<?> w = world; // workaround for Eclipse bug
      for (String name : w.getGridClasses())  
         try
         {
            gridClasses.add(loader.loadClass(name));
         }
         catch (Exception ex)
         {
            ex.printStackTrace();
         }

      Grid<?> gr = world.getGrid();
      gridClasses.add(gr.getClass());      
      
      makeNewGridMenu();      

      simControl = new SimController(this, display, displayMap, resources, loader);
      content.add(simControl.controlPanel(), BorderLayout.SOUTH);

      messageArea = new JTextArea(2, 40);
      messageArea.setEditable(false);
      messageArea.setFocusable(false);
      messageArea.setBackground(new Color(0xFAFAD2));
      content.add(new JScrollPane(messageArea), BorderLayout.NORTH);
      
      pack();
      setGrid(gr);
   }
   
   private void configureFileChooser()
   {
      fileChooser = new 
         JFileChooser()
         {
            public void approveSelection()
            {
               File f = getSelectedFile();
               if (f.isDirectory() && getFileView().getIcon(f) == null)               
                  setCurrentDirectory(f);
               else
                  super.approveSelection();
            }
         };
      fileChooser.setCurrentDirectory(new File("."));
      fileChooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
      final Icon worldIcon = new ImageIcon(getClass().getResource("World.gif"));
      
      final javax.swing.filechooser.FileFilter worldFilter = new 
         javax.swing.filechooser.FileFilter()
         {
            public boolean accept(File f)
            {
               return f.isDirectory() || isFileType(f, "class") || isFileType(f, "world");               
            }
            public String getDescription()
            {
               return resources.getString("chooser.type"); 
            }
         };

      fileChooser.setFileFilter(worldFilter);
      fileChooser.setFileView(new
            FileView()
            {
               public Icon getIcon(File f)
               {
                  if (f == null) return null;
                  if (f.isDirectory())
                  {
                     // Workaround against bug #5086412
                     File[] classFiles = f.listFiles(new 
                           java.io.FileFilter()
                           {
                              public boolean accept(File f) { return isFileType(f, "class"); }
                           });  
                     if (classFiles != null && classFiles.length > 0)
                        return worldIcon;
                  }
                  else if (isFileType(f, "world"))
                     return worldIcon;
                  return null;
               }
            });           
   }
   
   public void repaint()
   {
      String message = getWorld().getMessage();
      if (message == null)         
         message = resources.getString("message.default");      
      messageArea.setText(message);
      messageArea.repaint();
      display.repaint(); // for applet
      super.repaint();
   }

   public World getWorld()
   {
      return world;
   }

   public void setGrid(Grid newGrid)
   {
      Grid<?> oldGrid = world.getGrid();      
      Map<Location, Object> occupants = new HashMap<Location, Object>();
      for (Location loc : oldGrid.getOccupiedLocations())
         occupants.put(loc, world.remove(loc));

      world.setGrid(newGrid); // unavoidable warning
      for (Location loc : occupants.keySet())
      {
         if (newGrid.isValid(loc))
            world.add(loc, occupants.get(loc)); // unavoidable warning
      }
      
      display.setGrid(newGrid);
      repaint();
   }

   public void showError(Throwable t, String resource)
   {
      String text;
      try
      {
         text = resources.getString(resource + ".text");
      }
      catch (MissingResourceException e)
      {
         text = resources.getString("error.text");
      }

      String title;
      try
      {
         title = resources.getString(resource + ".title");
      }
      catch (MissingResourceException e)
      {
         title = resources.getString("error.title");
      }

      String reason = resources.getString("error.reason");
      String message = text + "\n" + MessageFormat.format(reason, new Object[]
         { t });

      JOptionPane.showMessageDialog(this, message, title,
            JOptionPane.ERROR_MESSAGE);
   }

   /**
    * Creates the drop-down menus on the frame.
    */

   private JMenu makeMenu(String resource)
   {
      JMenu menu = new JMenu();
      configureAbstractButton(menu, resource);
      return menu;
   }

   private JMenuItem makeMenuItem(String resource, ActionListener listener)
   {
      JMenuItem item = new JMenuItem();
      configureMenuItem(item, resource, listener);
      return item;
   }

   private void configureMenuItem(JMenuItem item, String resource,
         ActionListener listener)
   {
      configureAbstractButton(item, resource);
      item.addActionListener(listener);
      try
      {
         String accel = resources.getString(resource + ".accel");
         String metaPrefix = "@";
         if (accel.startsWith(metaPrefix))
         {
            int menuMask = getToolkit().getMenuShortcutKeyMask();
            KeyStroke key = KeyStroke.getKeyStroke(
                  KeyStroke.getKeyStroke(
                        accel.substring(metaPrefix.length()))
                        .getKeyCode(), menuMask);
            item.setAccelerator(key);
         }
         else 
         {
            item.setAccelerator(KeyStroke.getKeyStroke(accel));
         }
      }
      catch (MissingResourceException ex)
      {
         // no accelerator
      }
   }

   private void configureAbstractButton(AbstractButton button, String resource)
   {
      String title = resources.getString(resource);
      int i = title.indexOf('&');
      int mnemonic = 0;
      if (i >= 0)
      {
         mnemonic = title.charAt(i + 1);
         title = title.substring(0, i) + title.substring(i + 1);
         button.setText(title);
         button.setMnemonic(Character.toUpperCase(mnemonic));
         button.setDisplayedMnemonicIndex(i);
      }
      else
         button.setText(title);
   }

   private void makeMenus()
   {
      JMenuBar mbar = new JMenuBar();
      JMenu menu;

      menuItemsDisabledDuringRun = new ArrayList<JMenuItem>();

      mbar.add(menu = makeMenu("menu.file"));

      /*
      JMenuItem mItem;
      if (fileChooser != null)
      {
         menu.add(mItem = makeMenuItem("menu.file.load", new ActionListener()
            {
               public void actionPerformed(ActionEvent e)
               {
                  loadProject();
               }
            }));
         menuItemsDisabledDuringRun.add(mItem);
      }
      */
      newGridMenu = makeMenu("menu.file.new"); 
      menu.add(newGridMenu);
      menuItemsDisabledDuringRun.add(newGridMenu);

      menu.add(makeMenuItem("menu.file.quit", new ActionListener()
      {
         public void actionPerformed(ActionEvent e)
         {
            System.exit(0);
         }
      }));

      mbar.add(menu = makeMenu("menu.view"));

      menu.add(makeMenuItem("menu.view.up", new ActionListener()
            {
               public void actionPerformed(ActionEvent e)
               {
                  display.moveLocation(-1, 0);
               }
            }));
      menu.add(makeMenuItem("menu.view.down", new ActionListener()
            {
               public void actionPerformed(ActionEvent e)
               {
                  display.moveLocation(1, 0);
               }
            }));
      menu.add(makeMenuItem("menu.view.left", new ActionListener()
            {
               public void actionPerformed(ActionEvent e)
               {
                  display.moveLocation(0, -1);
               }
            }));
      menu.add(makeMenuItem("menu.view.right", new ActionListener()
            {
               public void actionPerformed(ActionEvent e)
               {
                  display.moveLocation(0, 1);
               }
            }));
      menu.add(makeMenuItem("menu.view.edit", new ActionListener()
            {
               public void actionPerformed(ActionEvent e)
               {
                  simControl.editLocation();
               }
            }));
      menu.add(makeMenuItem("menu.view.delete", new ActionListener()
            {
               public void actionPerformed(ActionEvent e)
               {
                  simControl.deleteLocation();
               }
            }));

      menu.add(makeMenuItem("menu.view.zoomin", new ActionListener()
      {
         public void actionPerformed(ActionEvent e)
         {
            display.zoomIn();
         }
      }));

      menu.add(makeMenuItem("menu.view.zoomout", new ActionListener()
      {
         public void actionPerformed(ActionEvent e)
         {
            display.zoomOut();
         }
      }));

      mbar.add(menu = makeMenu("menu.help"));
      menu.add(makeMenuItem("menu.help.about", new ActionListener()
      {
         public void actionPerformed(ActionEvent e)
         {
            showAboutPanel();
         }
      }));
      menu.add(makeMenuItem("menu.help.help", new ActionListener()
      {
         public void actionPerformed(ActionEvent e)
         {
            showHelp();
         }
      }));
      menu.add(makeMenuItem("menu.help.license", new ActionListener()
      {
         public void actionPerformed(ActionEvent e)
         {
            showLicense();
         }
      }));

      setRunMenuItemsEnabled(true);
      setJMenuBar(mbar);
   }

   private void makeNewGridMenu()
   {
      newGridMenu.removeAll();
      MenuMaker maker = new MenuMaker(this, resources, displayMap);
      maker.addGridConstructors(newGridMenu, gridClasses);
   }
   
   /**
    * Sets the enabled status of those menu items that are disabled when
    * running.
    */
   public void setRunMenuItemsEnabled(boolean enable)
   {
      for (JMenuItem item : menuItemsDisabledDuringRun)
         item.setEnabled(enable);
   }
   
   private void loadProject()
   {
      int r = fileChooser.showOpenDialog(this);
      if (r != JFileChooser.APPROVE_OPTION) return;
      WorldFrame newFrame = null;
      
      File f = fileChooser.getSelectedFile();
      URL url;
      if (isFileType(f, "world"))
      {
         try
         {
            url = f.toURL();
            ClassLoader loader = new URLClassLoader(new URL[] { url });            
            loadProject(url, loader, this);
         }
         catch (MalformedURLException e)
         {
            showError(e, "error.load.text");
         }
      }
      else if (f.isDirectory())
         newFrame = loadProject(f, null, this);
      else if (isFileType(f, "class"))
      {
         newFrame = loadProject(f.getParentFile(), stripExtension(f), this);
      }
      showNewFrame(newFrame);
   }
 
   private void showNewFrame(WorldFrame frame)
   {
      if (frame == null) return;      
      frame.setVisible(true);
      Rectangle r = getBounds();
      frame.setBounds(r.x + 10, r.y + 10, r.width, r.height);
   }
   
   /**
    * Loads a new world.
    * @param zipUrl the URL containing the world's classes
    * @param loader the class loader to use
    */
   public static WorldFrame loadProject(URL zipUrl, ClassLoader loader, WorldFrame frame)
   {
      Set<String> classNames = new TreeSet<String>();
      try
      {
         ZipInputStream zin = new ZipInputStream(zipUrl.openStream());
         ZipEntry entry;
         while ((entry = zin.getNextEntry()) != null)
         {
            String name = entry.toString();
            String suffix = ".class";
            if (name.endsWith(suffix))
            {
               String className = name.substring(0, name.length() - suffix.length());
               classNames.add(className);
            }
         }
      }
      catch (IOException e)
      {
         if (frame != null) frame.showError(e, "error.load.text");
         else e.printStackTrace();
      }
      return loadProject(classNames, loader, null, frame);
   }
   
   /**
    * Loads a new world.
    * @param dir the directory containing the world's classes
    * @param worldClassName the name of the world class
    */
   private static WorldFrame loadProject(File dir, String worldClassName, WorldFrame frame)
   {
      Set<String> classNames = new TreeSet<String>();
      File[] classFiles = dir.listFiles(new
            java.io.FileFilter()
            {
               public boolean accept(File f) { return isFileType(f, "class"); }
            });
      for (File f : classFiles)
      {
         String name = f.getName();
         String suffix = ".class";
         String className = name.substring(0, name.length() - suffix.length());
         classNames.add(className);
      }
      try
      {
         URL url = dir.toURL();
         ClassLoader loader = new URLClassLoader(new URL[] { url });
         return loadProject(classNames, loader, worldClassName, frame);
      }
      catch (MalformedURLException e)
      {
         if (frame != null) frame.showError(e, "error.load.text");
         else e.printStackTrace();
      }
      return null;
   }
   
   private static WorldFrame loadProject(Set<String> classNames, ClassLoader loader, String worldClassName, WorldFrame frame)
   {      
      Set<String> gridClassNames = new TreeSet<String>();
      Set<String> occupantClassNames = new TreeSet<String>();
      World newWorld = null;

      
      for (String name : classNames)
      {
         try
         {
            Class klass = loader.loadClass(name);
            if (World.class.isAssignableFrom(klass))
            {
               if (worldClassName == null || klass.getName().equals(worldClassName))
               newWorld = (World) klass.newInstance(); 
            }
            else if (Grid.class.isAssignableFrom(klass))
               gridClassNames.add(klass.getName());
            else
               occupantClassNames.add(klass.getName());
         }
         catch (ClassNotFoundException e)
         {
            if (frame != null) frame.showError(e, "error.load.text");
            else e.printStackTrace();
         }
         catch (IllegalAccessException e)
         {
            if (frame != null) frame.showError(e, "error.load.text");
            else e.printStackTrace();
         }
         catch (InstantiationException e)
         {
            if (frame != null) frame.showError(e, "error.load.text");
            else e.printStackTrace();
         }         
      }
      if (newWorld == null)
      {
         newWorld = new ActorWorld(); 
         for (String name : gridClassNames) newWorld.addGridClass(name);
         for (String name : occupantClassNames) 
         {
            newWorld.addOccupantClass(name);
            try
            {
               Class cl = loader.loadClass(name);
               if (Actor.class.isAssignableFrom(cl))
               {
                  Object occupant = cl.newInstance(); 
                  newWorld.add(occupant); // unavoidable warning
               }
            }
            catch (Exception ex) 
            {
               // we tried...
            }            
         }         
      }
      WorldFrame newFrame = new WorldFrame(newWorld, loader);
      return newFrame;
   }
   
   private static boolean isFileType(File f, String type)
   {
      return f.getName().toLowerCase().endsWith("." + type.toLowerCase());
   }
   
   private static String stripExtension(File f)
   {
      String name = f.getName();
      int i = name.lastIndexOf('.');
      if (i < 0) return name;
      else return name.substring(0, i);
   }
   
   /**
    * Brings up a simple dialog with some general information.
    */
   private void showAboutPanel()
   {
      String html = MessageFormat.format(resources
            .getString("dialog.about.text"), new Object[]
         { resources.getString("version.id") });
      JOptionPane.showMessageDialog(this, new JLabel(html), resources
            .getString("dialog.about.title"), JOptionPane.INFORMATION_MESSAGE);
   }

   /**
    * Brings up a window with a scrolling text pane that display the help
    * information for the simulation.
    */
   private void showHelp()
   {
      JDialog dialog = new JDialog(this, resources
            .getString("dialog.help.title"));
      final JEditorPane helpText = new JEditorPane();
      try
      {
         URL url = getClass().getResource("GridWorldHelp.html");

         helpText.setPage(url);
      }
      catch (Exception e)
      {
         helpText.setText(resources.getString("dialog.help.error"));
      }
      helpText.setEditable(false);
      helpText.addHyperlinkListener(new HyperlinkListener()
      {
         public void hyperlinkUpdate(HyperlinkEvent ev)
         {
            if (ev.getEventType() == HyperlinkEvent.EventType.ACTIVATED)
               try
               {
                  helpText.setPage(ev.getURL());
               }
               catch (Exception ex)
               {}
         }
      });
      JScrollPane sp = new JScrollPane(helpText);
      sp.setPreferredSize(new Dimension(650, 500));
      dialog.getContentPane().add(sp);
      dialog.setLocation(getX() + getWidth() - 200, getY() + 50);
      dialog.pack();
      dialog.setVisible(true);
   }

   private void showLicense()
   {
      JDialog dialog = new JDialog(this, resources
            .getString("dialog.license.title"));
      final JEditorPane text = new JEditorPane();
      try
      {
         URL url = getClass().getResource("GNULicense.txt");

         text.setPage(url);
      }
      catch (Exception e)
      {
         text.setText(resources.getString("dialog.license.error"));
      }
      text.setEditable(false);
      JScrollPane sp = new JScrollPane(text);
      sp.setPreferredSize(new Dimension(650, 500));
      dialog.getContentPane().add(sp);
      dialog.setLocation(getX() + getWidth() - 200, getY() + 50);
      dialog.pack();
      dialog.setVisible(true);
   }

   /**
    * Nested class that is registered as the handler for exceptions on the Swing
    * event thread. The handler will put up an alert panel, dump the stack trace
    * to the console, and then exit entire program rather than persist in an
    * inconsistent state, which would be the default behavior.
    */
   public static class GUIExceptionHandler
   {
      public void handle(Throwable e)
      {  
         JTextArea area = new JTextArea(10, 40);
         StringWriter writer = new StringWriter();
         e.printStackTrace(new PrintWriter(writer));
         area.setText(writer.toString());
         area.setCaretPosition(0);

         JOptionPane.showMessageDialog(null, new JScrollPane(area), e.toString(), JOptionPane.ERROR_MESSAGE);
         e.printStackTrace();
      }
   }
 }
